DEFDBL a-z
DECLARE FUNCTION f(x, y)
DECLARE SUB euler(x,y,dx,x1,y1)
DECLARE SUB midpoint(x,y,dx,x1,y1)
DECLARE SUB heun(x,y,dx,x1,y1)
DECLARE SUB runge(x,y,dx,x1,y1)
DECLARE FUNCTION yexact(x) 
'
' Sample program to solve the differential equation
'
'                   y' = f(x,y)
'
' using Euler's method, Midpoint method, Predictor-corrector method,
' and Runge-Kutta method of the fourth-order
'
' The exact form of the right hand side is placed in FUNCTION F
' of this program
PRINT ""
PRINT "P. Hellings: Astrophysics with a PC : SOME NUMERICAL METHODS"
PRINT "-------------------------------"
' Adapted by Kiyoshi Kawabata from the programs of Chapter 1
' on May 31, 2008
PRINT "Comparison between Euler's method, Midpoint method, " _ &
"Predictor-corrector method, and Runge-Kutta method of the 4th order"_&
" to solve the differential eq. y'=f(x,y)"
PRINT ""
COLOR 14
PRINT _
"For Table 1-1 on P.13, try x=0, y=1, dx=0.1, and number of iterations=18"
COLOR 15
INPUT "Initial condition  x(0) = "; x
INPUT "                   y(0) = "; y
INPUT "Stepsize           dx   = "; dx
INPUT "Number of iteration to be computed = "; n%
    ye=y: ym=y: yh=y: yr=y
    PRINT        "   i       x      y(Euler)   y(Mid pt) y(pre. cor)" &_
    "  y(Runge K)    y(exact)"
     PRINT USING "#### ####.#### ###.####### ###.####### ###.#######" &_
    " ###.####### ###.#######"; i%; x; ye; ym; yh; yr; yexact(x)
FOR i% = 1 TO n%

    CALL euler(x,ye,dx,x1,ye1)
    ye=ye1
    CALL midpoint(x,ym,dx,x1,ym1)
    ym=ym1
    CALL heun(x,yh,dx,x1,yh1)
    yh=yh1
    CALL runge(x,yr,dx,x1,yr1)
    yr=yr1
    
'  display new iteration
    PRINT USING "#### ####.#### ###.####### ###.####### ###.#######" &_
    " ###.####### ###.#######"; i%; x1; ye1; ym1; yh1; yr1; yexact(x1)

'  prepare for next iteration
    x = x1
    y = y1
    
NEXT i%
COLOR 14
PRINT "Press any key to terminate this  job!!!"
DO
LOOP WHILE INKEY$ = ""
END

SUB euler(x,y,dx,x1,y1)
'  Euler's method
    x1 = x + dx
    y1 = y + dx * f(x, y)   
END SUB

SUB midpoint(x,y,dx,x1,y1)
    
'  compute first half of new iteration
    x12 = x + .5 * dx
    y12 = y + .5 * dx * f(x, y)
    
'  compute complete iteration with slope at half the step
    x1 = x + dx
    y1 = y + dx * f(x12, y12)
END SUB   

SUB heun(x,y,dx,x1,y1)
 
'  compute the predicted new iteration
    x1p = x + dx
    y1p = y + dx * f(x, y)
         
'  compute the corrected new iteration
    x1 = x + dx
    y1 = y + .5 * dx * (f(x, y) + f(x1p, y1p))
END SUB    

SUB runge(x,y,dx,x1,y1)

'  compute the predicted new iteration
    x1 = x + dx

    k1 = dx * f(x, y)
    k2 = dx * f(x + .5 * dx, y + .5 * k1)
    k3 = dx * f(x + .5 * dx, y + .5 * k2)
    k4 = dx * f(x + dx, y + k3)
    
    y1 = y + (k1 + 2 * k2 + 2 * k3 + k4) / 6    
END SUB    

FUNCTION f (x, y)
'
'  contains the right hand side of the differential equation
'
    f = -2 * y * x
END FUNCTION

FUNCTION yexact(x)
'
'  exact solution for dy/dx=-2*y*x with y=0 at x=0
    yexact=EXP(-x^2)
END FUNCTION    
