DEFDBL a-z
DECLARE FUNCTION lg10 (x)
DECLARE SUB graphset
DIM      xs(501),fs(501),hs(501),plogs(501),dlogs(501),lms(501)
          
CONST g = 6.673E-08
      pi = 3.1415926536#
    
CLS
PRINT "Astrophysics with a PC : POLYTROPES"
PRINT "-----------------------------------"
PRINT " "
' Adapted by Kiyoshi Kawabata from the minimal solution program 
' of Chapter 7 on May 18, 2008
INPUT "0 for a new model, and 1 for the model of Table 7-2"; Nkey%
IF Nkey%=0 THEN
PRINT " "
PRINT "Input of initial conditions and model parameters : "
INPUT "polytrope index : ", n
INPUT "stepsize        : ", dr
INPUT "mass            : ", mass
INPUT "radius          : ", rad
ELSE 
     n=1.5: dr=0.05: mass=2.0: rad=3.0
ENDIF      
' show heading of main table of results on screen
PRINT "   i   x = r/rn       f           h       log(P/Pc)" &_
"   log(d/dc)      L*mr"

' compute polytrope results in the center of the star
x = 0
f = 1
h = 0
i% = 0
p = f ^ (n + 1)
d = f ^ n
m = -x * x * h

PRINT USING " ### ####.#### #####.##### #####.##### " &_
      "######.#### ######.#### ######.####";_
       i%; x; f; h; lg10(p); lg10(d); m
xs(i%)=x: fs(i%)=f: hs(i%)=h: plogs(i%)= lg10(p): dlogs(i%)=lg10(d): lms(i%)=m      

' compute first step
x = dr
f = 1! - x ^ 2 / 6 + x ^ 4 * n / 120
h = -x / 3 + x ^ 3 * n / 30
i% = 1
p = f ^ (n + 1)
d = f ^ n
m = -x * x * h
PRINT USING " ### ####.#### #####.##### #####.##### " &_
       "######.#### ######.#### ######.####"; i%; x; f; h;_
        lg10(p); lg10(d); m
xs(i%)=x: fs(i%)=f: hs(i%)=h: plogs(i%)= lg10(p): dlogs(i%)=lg10(d): lms(i%)=m
' initialize main cycle
i% = 2
verder% = 1

DO
' this is the main cycle
    x12 = x + .5 * dr
    f12 = f + .5 * dr * h

'   check whether f12 ( = F at half step) is still positive
    IF f12 > 0! THEN
        h12 = h + .5 * dr * (-f ^ n - 2 * h / x)
        x1 = x + dr
        f1 = f + dr * h12
'       check whether f1 ( = F at new state) is still positive
        IF f1 > 0! THEN
            h1 = h + dr * (-f12 ^ n - 2 * h12 / x12)   
            
'           compute pressure, density and mass            
            p = f1 ^ (n + 1)
            d = f1 ^ n
            m = -x1 * x1 * h1
            Nlay%=i%
'           show results on screen
            PRINT USING " ### ####.#### #####.##### " &_
            "#####.##### ######.#### ######.#### ######.####";_
             i%; x1; f1; h1; lg10(p); lg10(d); m
            xs(i%)=x1: fs(i%)=f1: hs(i%)=h1
            plogs(i%)= lg10(p): dlogs(i%)=lg10(d): lms(i%)=m
'           prepare for next iteration
            i% = i% + 1
            x = x1
            f = f1
            h = h1
           
'          pause every 15 iterations
            IF i% MOD 15 = 0 THEN
                PRINT "Press any key to continue"
                DO
                LOOP WHILE INKEY$ = ""
            END IF
        ELSE
'          this ELSE is reached if f1 was negative
            verder% = 0
        END IF
    ELSE
'       this ELSE is reached if f12 was negative
        verder% = 0
    END IF
LOOP UNTIL verder% = 0

' compute general characteristics and surface data
xm = x - f / h
hm = h + (xm - x) * (-f ^ n - 2 * h / x)
fm = 0
pc = 9.048E+14 * mass * mass / (n + 1) / hm / hm / rad ^ 4
dm = 1.42 * mass / rad ^ 3
dc = -dm * xm / 3 / hm
lam = -xm * xm * hm / mass
rn = rad / xm

' show general characteristics and surface data on screen
PRINT " "
PRINT "central pressure (Pc) : "; pc
PRINT "average density       : "; dm
PRINT "central density (dc)  : "; dc
PRINT "mass parameter (L)    : "; lam
PRINT "distance unit (rn)    : "; rn
PRINT USING "x-final               : #####.#### "; xm
PRINT USING "-x2*h (final)         : #####.#### "; _
                                                  -xm * xm * hm
'  PRINT curdir$                                                
 PRINT ""
 LOCATE (25,1)
 COLOR 14: PRINT "Press any key to proceed to a model plot"
 COLOR 7
 DO
 LOOP WHILE INKEY$ = ""
 CALL graphset 
 ' (1) x vs. F(x)
 PSET (xs(0),fs(0))
 FOR i%=1 TO Nlay%
     LINE -(xs(i%),fs(i%)),2
 NEXT i%   
 ' (2) x vs. H(x)+1
 PSET (xs(0),hs(0)+1.0)
 FOR i%=1 TO Nlay%
     LINE -(xs(i%),hs(i%)+1.0), 3
 NEXT i%
 ' (3) x vs. 1+log P/max|log P|
 plogmax=ABS(plogs(Nlay%))
 PSET (xs(0),plogs(0)/plogmax+1.0)
 FOR i%=1 TO Nlay%
     LINE -(xs(i%),plogs(i%)/plogmax+1.0), 4
 NEXT i%
' (4) x vs. LM/max(LM)
 PSET (xs(0),lms(0)/lms(Nlay%))
 FOR i%=1 TO Nlay%
     LINE -(xs(i%),lms(i%)/lms(Nlay%)), 1
 NEXT i%
 
     Imid%=Nlay%*0.8
     IX%=PMAP(xs(Imid%),0)/8
     IY%=PMAP(fs(Imid%)+0.1,1)/16+1
     LOCATE (IY%,IX%)
     PRINT "F(x)"
     
     IX%=PMAP(xs(Nlay%),0)/8+4
     IY%=PMAP(hs(Imid%)+1,1)/16
     LOCATE (IY%,IX%)
     PRINT "H(x)+1"
     
     PSET(xs(Imid%),plogs(Imid%)/plogmax+1)
     LINE -STEP(0.4,0.1)
     IX%=PMAP(xs(Imid%)+0.4,0)/8+1
     IY%=PMAP(plogs(Imid%)/plogmax+1+0.1,1)/16+1
     LOCATE (IY%,IX%)
     PRINT "(log P/Pc)/max.(log P/Pc)+1"
     
     IX%=PMAP(xs(0.3*Nlay%),0)/8+7
     IY%=PMAP(lms(0.3*Nlay%)/lms(Nlay%),1)/16+2
     LOCATE (IY%,IX%)
     PRINT "LM(x)/max.LM(x)"     
LOCATE (29,2)
COLOR 14: PRINT "Press any key to exit"
COLOR 7
DO
LOOP WHILE INKEY$ = ""
END

FUNCTION lg10 (x)
'
'  computes the base 10 logarithm of x 
'  using the natural logarithm LOG(x)
'
     lg10 = LOG(x) / LOG(10!)
END FUNCTION

SUB GRAPHSET
  
        SCREEN 18,2
        __SCREENSET 1,0
        WINDOW  (-0.2,1.8)-(5.0,-0.05)
        VIEW    (20,20)-(650,450)
    
    LINE (0.0,-0.2)-(0.0,1.2),5
    LINE (-0.2,0.0)-(4.5,0.0),5
    LINE (0.0,1.0)-(4.5,1.0),5
    
    IY%=PMAP(-0.05,1)/16+1
    FOR i%=1 TO 4
       x=i%    
    LINE (x,0.0)-(x,0.03),5
    IX%=PMAP(i%,0)/8+2
    LOCATE(IY%,IX%)
    PRINT i%
    NEXT I%
    IX%=PMAP(-0.05,0)/8+1
    FOR i%=1 TO 5 
       y=0.2*i%
    LINE (0.0,y)-(0.1,y),5
    IY%=PMAP(y,1)/16+2
    LOCATE (IY%,IX%)
    PRINT USING "#.#";y
    NEXT i%
    
    IX%=PMAP(4.7,0)/8+1
    IY%=PMAP(-0.025,1)/16+2
    LOCATE (IY%,IX%)
    PRINT "X"
END SUB