DEFDBL a-z
DECLARE FUNCTION radpress (t)
DECLARE FUNCTION temp (tau, teff)
DECLARE FUNCTION absorp (t, d)
DECLARE FUNCTION dens (t, p, mu)
DECLARE FUNCTION geffect (gs, teff, k, tau)
DECLARE SUB stap (teff, gs, mu, dtau, tau, t, pg, d, kk, ge, kk1, d1)
DECLARE SUB disp (i%, tau, t, pg, pr, rho, kappa, ge, z)
DECLARE SUB graphset(Xmax,Ymax)

Dim teffs(2), gss(2), mus(2)
Dim taus(99), ts(99), pgs(99), prs(99), ds(99), kks(99)
Dim ges(99),  zs(99)
COMMON SHARED pi, g, a, r

' some physical constants used in the program
pi = 3.1415926535#
g = 6.673E-08
a = 7.56464E-15
r = 8.314E+07

CLS
PRINT " "
PRINT "P.Hellings: Astrophysics with a PC : STELLAR ATMOSPHERE"
PRINT "-------------------------------------------"
PRINT " "
' Adapted by Kiyoshi Kawabata from the program of Chapter 9
' on May 23, 2008

INPUT "0 for a new model, and 1 for the model of Table 7-2"; Nkey%
IF Nkey%=0 THEN
PRINT " "
PRINT "Input of initial conditions and model parameters : "
INPUT "Effective temperature         : ", teff
INPUT "Surface gravit.accel.(log10)  : ", gs
INPUT "Average mean molecular weight : ", mu
gs = 10 ^ gs
     teffs(0)=teff: gss(0)=gs: mus(0)=mu
ELSE 
      FOR i%=1 TO 2
      READ teffs(i%),gss(i%),mus(i%)
           gss(i%)=10^gss(i%)
      NEXT i%
      
ENDIF  
   teff=teffs(Nkey%): gs=gss(Nkey%): mu=mus(Nkey%)
   
IF Nkey% >=1 THEN 
PRINT ""    
PRINT "Input of initial conditions and model parameters : "
PRINT "Effective temperature         : ", teff
PRINT "Surface gravit.accel.(log10)  : ", gs
PRINT "Average mean molecular weight : ", mu
ENDIF

' display heading of main table
PRINT " "
PRINT "  i    tau      T      Pg     Pr       d       " & _
      "k       ge       z(km)"

' compute central values of physical quantities
tau = 0
d = 1E-13
t = temp(tau, teff)
pg = r * d * t / mu
pr = radpress(t)
kk = absorp(t, d)
ge = geffect(gs, teff, kk, tau)
z = 0
i% = 0
CALL disp(i%, tau, t, pg, pr,d, kk, ge, z)
  taus(i%)=tau: ts(i%)=t: pgs(i%)=pg: prs(i%)=pr: ds(i%)=d
  kks(i%)=kk: ges(i%)=ge: zs(i%)=z

' compute physical quantities in the layer 1
i% = 1
dtau = .001
CALL stap(teff, gs, mu, dtau, tau, t, pg, d, kk, ge, kk1, d1)
    z = dtau / kk1 / d1
    pr = radpress(t)
CALL disp(i%, tau, t, pg, pr, d, kk, ge, z)    
  taus(i%)=tau: ts(i%)=t: pgs(i%)=pg: prs(i%)=pr: ds(i%)=d
  kks(i%)=kk: ges(i%)=ge: zs(i%)=z
' next FOR-cycle computes the other layers
FOR i% = 2 TO 32
    dtau = .25 * tau
    CALL stap(teff, gs, mu, dtau, tau, t, pg, d, kk, ge, kk1, d1)
    dz = dtau / kk1 / d1
    pr = radpress(t)
    z = z + dz
CALL disp(i%, tau, t, pg, pr, d, kk, ge, z)    
  taus(i%)=tau: ts(i%)=t: pgs(i%)=pg
  : prs(i%)=pr: ds(i%)=d:
  kks(i%)=kk: ges(i%)=ge: zs(i%)=z
  Nlay%=i%  
' pause after having shown layer 15    
    IF i% = 15 THEN
        PRINT "press any key to continue"
        DO
        LOOP WHILE INKEY$ = ""
    END IF
NEXT i%

COLOR 14
PRINT " "
PRINT "Model complete. Press any key to proceed to model plotting"
DO
LOOP WHILE INKEY$ = ""
     Xmax=1.1
     Ymax=1.1
CALL graphset(Xmax,Ymax)
' (1) tau vs. T/Tmax
      tmax=ts(Nlay%)
    PSET (0,ts(0)/tmax)
    FOR i%=1 TO Nlay%
     LINE -(taus(i%),ts(i%)/tmax),2
    NEXT i%   
    Nmid%=0.8*Nlay% 
'    PRINT "Nmid%=";Nmid%;"   Nlay%=";Nlay%
    IX%=PMAP(taus(Nmid%),0)/8-7
    IY%=PMAP(ts(Nmid%)/tmax,1)/16+1
    LOCATE (IY%,IX%): PRINT "T/Tmax"
'(2) tau vs. Pg/Pgmax
    Nmid%=0.9*Nlay%
    pgmax=pgs(Nlay%)
    PSET (0,pgs(0)/pgmax)
    FOR i%=1 TO Nlay%
     LINE -(taus(i%),pgs(i%)/pgmax),1
    NEXT i%   
    IX%=PMAP(taus(Nmid%),0)/8
    IY%=PMAP(pgs(Nmid%)/pgmax,1)/16+3
    LOCATE (IY%,IX%): PRINT "Pg/Pgmax"
'(3) tau vs. Pg/Pgmax
    Nmid%=0.95*Nlay%
    prmax=prs(Nlay%)
    PSET (0,prs(0)/prmax)
    FOR i%=1 TO Nlay%
     LINE -(taus(i%),prs(i%)/prmax),4
    NEXT i%   
    LINE (taus(25),prs(25)/prmax)-(taus(25)+0.1,prs(25)/prmax-0.07)
    IX%=PMAP(taus(25)+0.1,0)/8+2
    IY%=PMAP(prs(25)/prmax-0.07,1)/16+2
    LOCATE (IY%,IX%): PRINT "Pr/Prmax"
'(4) tau vs. d/dmax
    Nmid%=0.95*Nlay%
    dmax=ds(Nlay%)
    PSET (0,ds(0)/dmax)
    FOR i%=1 TO Nlay%
     LINE -(taus(i%),ds(i%)/dmax),3
    NEXT i%   
    LINE(taus(25),ds(25)/dmax)-(taus(25)+0.1,ds(25)/dmax-0.1)
    IX%=PMAP(taus(25)+0.11,0)/8+2
    IY%=PMAP(ds(25)/dmax-0.11,1)/16+2
    LOCATE (IY%,IX%): PRINT "d/dmax"    
    
 '(5) tau vs. k/kmax
    Nmid%=0.95*Nlay%
    kmax=kks(Nlay%)
    PSET(0,kks(0)/kmax)
    FOR i%=1 TO Nlay%
      LINE -(taus(i%), kks(i%)/kmax),14
    NEXT i%   
    LINE(taus(25),kks(25)/kmax)-(taus(25)+0.1,kks(25)/kmax-0.1)
    IX%=PMAP(taus(25)+0.11,0)/8+2
    IY%=PMAP(kks(25)/kmax-0.11,1)/16+2
    LOCATE (IY%,IX%): PRINT "k/kmax"  
        
 '(6) tau vs. k/kmax
    Nmid%=0.95*Nlay%
    gemax=ges(0)
    PSET(0,ges(0)/gemax)
    FOR i%=1 TO Nlay%
      LINE -(taus(i%), ges(i%)/gemax),11
    NEXT i%   
    LINE(taus(23),ges(23)/gemax)-(taus(23),ges(23)/gemax-0.02)
    IX%=PMAP(taus(23),0)/8+2
    IY%=PMAP(ges(23)/gemax-0.02,1)/16+2
    LOCATE (IY%,IX%): PRINT "ge/gemax"    
    
 '(7) tau vs. k/kmax
    Nmid%=0.95*Nlay%
    zmax=zs(Nlay%)
    PSET(0,zs(0)/zmax)
    FOR i%=1 TO Nlay%
      LINE -(taus(i%), zs(i%)/zmax),12
    NEXT i%   
    LINE(taus(27),zs(27)/zmax)-(taus(27)+0.06,zs(27)/zmax-0.05)
    IX%=PMAP(taus(27)+0.06,0)/8+2
    IY%=PMAP(zs(27)/zmax-0.05,1)/16+2
    LOCATE (IY%,IX%): PRINT "z"  
    
    LOCATE (10,55):    PRINT USING "Tmax= #####.#";tmax
    LOCATE (11,55):    PRINT USING "Pgmax=  ###.##";pgmax 
    LOCATE (12,55):    PRINT USING "Prmax=  ###.##";prmax
    LOCATE (13,55):    PRINT USING "dmax=    ##.###";dmax*1.e11
    LOCATE (14,55):    PRINT USING "kmax=    ##.#####";kmax
    LOCATE (15,55):    PRINT USING "gemax= ####.##";gemax
    LOCATE (16,55):    PRINT USING "zmax=######.#";zmax/100000!
  __SLEEP
  
    COLOR 15
' Sample input data for an A0 dwarf(e.g., Serius) and 
'                   and an A0 supergiant (Deneb)
'            teff,  gs,  mu
     DATA 10000.0, 4.0, 1.048
     DATA 10000.0, 2.0, 0.738
END


FUNCTION absorp (t, d)
'
' computes the absorption coefficient for given temperature t 
' and density d
     absorp = 1.984E+24 * d / t ^ 3.5
END FUNCTION


FUNCTION dens (t, p, mu)
'
' computes the density for input values of temperature t, 
' pressure p and mu
     dens = p * mu / t / r
END FUNCTION

SUB disp (i%, tau, t, pg, pr, d, kk, ge, z)
'
' this SUB is called each layer to display the results
' for that layer

PRINT USING "### ##.##### #####.# ###.## ###.## "  &_
"###.### ##.##### #####.## #######.#"; _
i%; tau; t; pg; pr; d * 1E+11; kk; ge; z / 100000!
END SUB

FUNCTION geffect (gs, teff, k, tau)
'
' computes the effective gravitational acceleration
'
    geffect = gs - k * a * teff ^ 4 _
             / 4 * (1 + .459 * EXP(-3.4488 * tau))
END FUNCTION


FUNCTION radpress (t)
'
'  computes the radiation pressure for given 
'  temperature T and density d
     radpress = a / 3 * t ^ 4
END FUNCTION

SUB _
stap (teff, gs, mu, dtau, tau, t, pg, d, kk, ge, kk1, d1)
'
'  computes the physical quantities at layer i+1 
'  starting from layer i
    tau1 = tau + .5 * dtau
    t1 = temp(tau1, teff)
    pg1 = pg + .5 * dtau * ge / kk
    d1 = dens(t1, pg1, mu)
    kk1 = absorp(t1, d1)
    ge1 = geffect(gs, teff, kk1, tau1)
    tau = tau + dtau
    t = temp(tau, teff)
    pg = pg + dtau * ge1 / kk1
    d = dens(t, pg, mu)
    kk = absorp(t, d)
    ge = geffect(gs, teff, kk, tau)
END SUB

FUNCTION temp (tau, teff)
'
'  computes the temperature for given optical depth tau
'  and effective temperature teff
     q = .7104 - .1331 * EXP(-3.4488 * tau)
     temp = teff * (.75 * (tau + q)) ^ .25
END FUNCTION

SUB graphset(Xmax,Ymax)
     SCREEN 18,2
        __SCREENSET 1,0
 
        WINDOW  (-0.05,Ymax)-(Xmax,-0.05)
        VIEW    (15,15)-(650,450)
       
    Nx%=10*Xmax
    Ny%=10*Ymax
    LINE (0,Ymax)-(0,0),5
    LINE (0,1)-(0.95*Xmax,1),5
    LINE (0,0)-(0.95*Xmax,0),5
    
 '   IY%=PMAP(0.3,1)/16+1
  '  FOR i%=2 TO Nx% STEP 2
   ' LINE (i%,0)-(i%,-0.3),5
  '  LINE (i%,-Ymax)-(i%,-Ymax+0.3),5
  '  IX%=PMAP(i%,0)/8+2
 '   LOCATE(IY%,IX%)
 '   PRINT i%;
 '   NEXT I%
    IX%=PMAP(-0.025,0)/8+1
    FOR i%=0 TO Ny% STEP 2 
       y=0.1*i%
    LINE (0.0,y)-(0.025,y),5
    IY%=PMAP(y,1)/16+2
    LOCATE (IY%,IX%)
    PRINT USING "#.#";y;
    NEXT i%
    IY%=PMAP(-0.1,1)/16
    FOR i%=2 TO Nx% STEP 2
      x=0.1*i%
    LINE(x,0)-(x,0.025), 5
    IX%=PMAP(x,0)/8+2
    LOCATE (IY%,IX%): PRINT USING "#.#";x
    NEXT i%
     
    IX%=PMAP(0.95*Xmax,0)/8+1
    IY%=PMAP(0,1)/16
    LOCATE (IY%,IX%)
    PRINT "Tau"   
END SUB    

SUB DOTS(x1,y1,x2,y2)
    dx=SGN(x2-x1)*0.05
    dy=SGN(y2-y1)*0.05
    IF dx <> 0.0 THEN
    NX%=INT((x2-x1)/dx+1)
    ELSE 
    NY%=INT((y2-y1)/dy+1)
    ENDIF
'    PRINT "x2-x1="; x2-x1; "  y2-y1=";y2-y1 
    N%=NX%
    IF NY% > NX% THEN N%=NY%
'   PRINT "N%=";N%;"  dx=";dx;"  dy=";dy
    FOR I%=2 TO N%  STEP 2 
    xb=x1+dx*(I%-1)
    yb=y1+dy*(I%-1)
    LINE (xb-dx,yb-dy)-(xb,yb),5
    NEXT I%
END SUB