DEFDBL a-z
DECLARE FUNCTION f (x)
DECLARE FUNCTION lg10 (x)
DECLARE FUNCTION dfdx (x)
DECLARE SUB density (p, xold, rho, xnew)
DECLARE FUNCTION press (m, r, d)
DECLARE FUNCTION mass (d, r)
DECLARE SUB graphset(Xmax,Ymax)

Dim rs(101),ms(101),plogs(101),dlogs(101),xxs(101)

COMMON SHARED g, m0, r0, a, pi

' some physical quantities used in the program
g = 6.673E-08
m0 = 2E+33
r0 = 6.96E+10
a = 6.01E+22
pi = 3.1415926536#

CLS
PRINT "P.Hellings: Astrophysics with a PC : WHITE DWARF"
PRINT "------------------------------------"
PRINT " "
' Adapted by Kiyoshi Kawabata from the program of Chapter 10 
' on May 24.
PRINT " "
PRINT "Input of initial conditions and model parameters  : "
INPUT "Log10 of the central density (Key in 8 for Table 10-2) : ", rhc
INPUT "stepsize in km               (Key in 80 for Table 10-2): ", drkm

' central values of the physical variables
rhoc = 10 ^ rhc
dr = 100000! * drkm
PRINT " "
    
m = 0!
r = 0!
xc = (rhoc / 1964000!) ^ (1 / 3)
pc = a * f(xc)
i% = 0

' display heading of the table with the results
PRINT "  i     r          Mr        log(P)    log(rho)      x"

' display central values (layer zero)
PRINT USING "###  ###.#####  ###.#####  ###.#####  " &_  
"###.#####  ##.#######";_
 i%; r / r0; m / m0; lg10(pc); lg10(rhoc); xc
  rs(i%)=r/r0: ms(i%)=m/m0: plogs(i%)=lg10(pc):
  dlogs(i%)=lg10(rhoc): xxs(i%)=xc
' compute first step from layer zero to layer one
' and show them on screen
p = pc - 2 / 3 * g * pi * (rhoc * dr) ^ 2
m = 4 / 3 * pi * rhoc * dr ^ 3
CALL density(p, xc, d, x)
r = dr
i% = 1
PRINT USING "###  ###.#####  ###.#####  ###.#####  " &_
            "###.#####  ##.#######";_
 i%; r / r0; m / m0; lg10(p); lg10(d); x
  rs(i%)=r/r0: ms(i%)=m/m0: plogs(i%)=lg10(p): 
  dlogs(i%)=lg10(d): xxs(i%)=x   
' prepare for the other layers(from layer two to the surface)
surface% = 0
i% = 2

' here starts the main cycle for the other layers
DO
    r1 = r + .5 * dr
    p1 = p + .5 * dr * press(m, r, d)

' check if pressure (half step i+1/2) is still positive
' otherwise surface is reached
    IF p1 > 0 THEN
        m1 = m + .5 * dr * mass(d, r)
        CALL density(p1, x, d1, x1)
        r = r + dr
        p = p + dr * press(m1, r1, d1)

'       check if pressure (full step i+1) is still positive
'       otherwise surface is reached
        IF p > 0 THEN
            m = m + dr * mass(d1, r1)
            CALL density(p, x1, d, x)

'          show results of newly computed layer on screen            
            PRINT USING "###  ###.#####  ###.#####  " &_
            "###.#####  ###.#####  ##.#######"; _
            i%; r / r0; m / m0; lg10(p); lg10(d); x
            rs(i%)=r/r0: ms(i%)=m/m0: plogs(i%)=lg10(p): 
            dlogs(i%)=lg10(d): xxs(i%)=x 
            Nlay%=i%
            i% = i% + 1

'          pause after every 12 layers
            IF i% MOD 12 = 0 THEN
                PRINT "Press any key to continue"
                DO
                LOOP WHILE INKEY$ = ""
            END IF
        ELSE    'this ELSE is reached if pressure (i+1)
                'was negative
            surface% = 1
        END IF
    ELSE        'this ELSE is reached if pressure (i+1/2)
                'was negative
        surface% = 1
    END IF
LOOP UNTIL surface% = 1
    
' show general results on screen
PRINT " "
PRINT USING "Mtot/M0    : ##.####  (total stellar mass in units of M0)"; _
            m / m0
PRINT USING "log Pc     : ##.####";plogs(0)
PRINT USING "log dc     : ##.####";dlogs(0)
PRINT USING "xc         : ##.####";xxs(0)
PRINT USING "log R/R0   : ##.####  (stellar radius in units of R0)";_
lg10(rs(Nlay%)) ' log of stellar radius 
COLOR 14
PRINT "Press any key to proceed to model plottting"
DO
LOOP WHILE INKEY$ = ""
     COLOR 15 
     Xmax=1.1*rs(Nlay%)
     Ymax=1.25
     CALL graphset(Xmax,Ymax)
' (1) r vs. M/Mmax
      mmax=ms(Nlay%)
    PSET (0,ms(0)/mmax)
    FOR i%=1 TO Nlay%
     LINE -(rs(i%),ms(i%)/mmax),2
    NEXT i%   
    Nmid%=0.3*Nlay% 
'    PRINT "Nmid%=";Nmid%;"   Nlay%=";Nlay%
    IX%=PMAP(rs(Nmid%),0)/8-2
    IY%=PMAP(ms(Nmid%)/mmax,1)/16+1
    LOCATE (IY%,IX%): PRINT "M/Mtot"
' (2) r vs. log Pg/log Pg(0)
    Nmid%=0.4*Nlay%
    logpmax=plogs(0)
    PSET (0,plogs(0)/logpmax)
    FOR i%=1 TO Nlay%
     LINE -(rs(i%),plogs(i%)/logpmax),1
    NEXT i%   
    IX%=PMAP(rs(Nmid%),0)/8
    IY%=PMAP(plogs(Nmid%)/logpmax,1)/16+1
    LOCATE (IY%,IX%): PRINT "log P/log Pc"
' (3) r vs. log d/log d(0)
    Nmid%=0.9*Nlay%
    logdmax=dlogs(0)
    PSET (0,dlogs(0)/logdmax)
    FOR i%=1 TO Nlay%
    LINE -(rs(i%),dlogs(i%)/logdmax),4
    NEXT i%   
    IX%=PMAP(rs(Nmid%),0)/8+1
    IY%=PMAP(dlogs(Nmid%)/logdmax,1)/16+6
    LOCATE (IY%,IX%): PRINT "log d/log dc"
'(4) r vs. x/x(0)
    Nmid%=0.7*Nlay%
    xc=xxs(0)
    PSET (0,xxs(0)/xc)
    FOR i%=1 TO Nlay%
     LINE -(rs(i%),xxs(i%)/xc),3
    NEXT i%   
'    LINE(rs(Nmid%),xxs(Nmid%)/xmax)-(rs(25)+0.1,xxs(25)/xmax-0.1)
    IX%=PMAP(rs(Nmid%),0)/8+10
    IY%=PMAP(xxs(Nmid%)/xmax,1)/16+3
    LOCATE (IY%,IX%): PRINT "x/xc"    
    
    COLOR 14: LOCATE (30,50)
    PRINT "Any key to terminate this job"
    __SLEEP
END

SUB density (p, xold, rho, xnew)
'
'  computes the density rho and the corresponding
'  value xnew from the 
'  input pressure P and starting guess xold

     k = p / a
     stopcrit% = 0
     DO
           xnew = xold - (f(xold) - k) / dfdx(xold)
           IF ABS(xold - xnew) < .000001 THEN
               stopcrit% = 1
           ELSE
               xold = xnew
           END IF
     LOOP UNTIL stopcrit% = 1
     rho = 1964000! * xnew ^ 3
END SUB

FUNCTION dfdx (x)
'
'  evaluates the derivative of f(x)
'
     dfdx = 8 * x ^ 4 / SQR(1 + x ^ 2)
END FUNCTION

FUNCTION f (x)
'
'  evaluates the function f(x)
'
     f = x * (2 * x ^ 2 - 3) * SQR(1 + x ^ 2) +   _
         3 * LOG(x + SQR(1 + x ^ 2))
END FUNCTION

FUNCTION lg10 (x)
'
'  computes the base 10 logarithm of x using the natural
'  logarithm LOG(x)
'
    lg10 = LOG(x) / LOG(10!)
END FUNCTION

FUNCTION mass (d, r)
'
'  computes the right hand side of the differential equation
'  of mass continuity
'
     mass = 4 * pi * d * r * r
END FUNCTION

FUNCTION press (m, r, d)
'
'  computes the right hand side of the equation of hydro-
'  static equilibrium
'
    press = -g * m * d / r ^ 2
END FUNCTION


SUB graphset(Xmax,Ymax)
     SCREEN 18,2
        __SCREENSET 1,0
        Xoff=Xmax/15.0
        Yoff=-Ymax/40.0
        WINDOW  (-0.0003,Ymax)-(Xmax,-0.03)
        VIEW    (25,25)-(650,450)
       
    Nx%=1000*Xmax
    Ny%=10*Ymax
 
    LINE (0,1)-(0.95*Xmax,1),5
    LINE (0,0)-(0.95*Xmax,0),5
    LINE (0,0)-(0,Ymax),5
 
    IX%=PMAP(-Xoff*0.4,0)/8+1
    FOR i%=0 TO Ny%-1 STEP 2 
       y=0.1*i%
    LINE (0.0,y)-(0.00012,y),5
    IY%=PMAP(y,1)/16+2
    LOCATE (IY%,IX%)
    PRINT USING "#.#";y;
    NEXT i%

    IY%=PMAP(-0.05,1)/16+2
 '   IF IY% > 25 THEN IY%=25
    FOR i%=1 TO Nx%-1 
      x=i%/1000.0
    LINE(x,0)-(x,0.025), 5
    IX%=PMAP(x,0)/8+3
    LOCATE (IY%,IX%): PRINT USING "#";i%
    NEXT i%
     
    IX%=PMAP(0.95*Xmax,0)/8+1
    IY%=PMAP(0,1)/16+3
    LOCATE (IY%,IX%)
    PRINT "1000*r"   
END SUB    
